#!/bin/bash

########################################################################
# SLURM Header
########################################################################

#SBATCH -J QE
#SBATCH --nodes=4
#SBATCH -n 512
#SBATCH -p ROME-7742_hdr100_256gb_3200
#SBATCH --exclusive
##SBATCH -L scratch_lustre_ddn7990*1
#SBATCH -A rfp_2020_izum

#SBATCH --time=0:20:00

source /software/modulefiles/load_me.sh

SCRIPT_ADMIN=/home_nfs/script/admin

function fatal_error() {
  echo "Fatal error $*"
  exit 1
}

########################################################################
# Setup zone reclaim parameter and check that clush works
########################################################################

  if [ -d "${SCRIPT_ADMIN}" ]
  then

    #-----------------------------------------------------
    #
    # Set Zone reclaim policy to 1 (prioritize computations to IO)
    #
    clush -bw $SLURM_NODELIST " sudo ${SCRIPT_ADMIN}/zone_reclaim 1"


    if [ $? -ne 0 ]
    then
    echo "cannot clush on all nodes!"
    exit
    fi
  else
    echo "not in BULL environment, cannot do special tricks!"
  fi

########################################################################
# Get benchmark study parameters
########################################################################

  FREQ=$1
  RUN_TYPE=$2
  my_omp=$3

  if [ -z "$FREQ" ]
  then
#    FREQ=2400000
    FREQ=2250000
  fi


  if [ -z "$RUN_TYPE" ]
  then
    RUN_TYPE=normal
  fi


########################################################################
# Setup frequency
########################################################################
  if [ -d "${SCRIPT_ADMIN}" ]
  then
    #enable THP
    clush -bw $SLURM_NODELIST "sudo ${SCRIPT_ADMIN}/enable_transparent_hugepages_rhel7"
    if test "$FREQ" = "TURBO"
    then
      clush -bw $SLURM_NODELIST "sudo ${SCRIPT_ADMIN}/enable_boost_amd.sh"
    else
      clush -bw $SLURM_NODELIST "sudo ${SCRIPT_ADMIN}/pwrb -f $FREQ"
      clush -bw $SLURM_NODELIST " sudo ${SCRIPT_ADMIN}/set_uncore_freq 30"
    fi
  fi
  export FREQ=$(  cat /sys/devices/system/cpu/cpu*/cpufreq/scaling_cur_freq | uniq )


########################################################################
# Source benchmark environment
########################################################################

  source ${SLURM_SUBMIT_DIR}/env_bench
#  source ${BUILD_DIR}/install/bin/GMXRC.bash
#  FS=/scratch_lustre_ddn7990/$USER/WORKDIR/QE
  #FS=${SLURM_SUBMIT_DIR}/WORKDIR/

FS=/scratch_na/users/bpajotb/2020_IZUM/QE

########################################################################
# Prepare benchmark topology for runtype
########################################################################

  nodes=${SLURM_NNODES}
  tasks=${SLURM_NTASKS}

  CORES_PER_SOCKET=`LANG=C lscpu|grep "Core(s) per socket:" |awk '{ print $NF }'`
  SOCKETS=`LANG=C lscpu|grep "Socket(s):" |awk '{ print $NF }'`

function skip()
{
  if [ -z "$my_omp" ]
  then
    my_omp=4
  fi
  export OMP_NUM_THREADS=$my_omp
}

  cores_per_node=$(( CORES_PER_SOCKET*SOCKETS ))
  ppn=$(( tasks/nodes ))
  threads=$(( cores_per_node/ppn ))

  if [ "$RUN_TYPE" == "normal" ]
  then
    echo "RUN_MASK=gen_cpu_mask_amd.py -s 2 -c $cores_per_node -t $threads -p ${ppn} --normal"
    RUN_MASK=`gen_cpu_mask_amd.py -s 2 -c $cores_per_node -t $threads -p ${ppn} --normal`
    NUMACTL=
    MEMBW=1
    IBBW=1
  elif [ "$RUN_TYPE" == "hyperthread" ]
  then
    threads=$[threads *2]
    RUN_MASK=`gen_cpu_mask.py -s 2 -c $cores_per_node -t $threads -p ${ppn} --hyperthread`
    NUMACTL=
    MEMBW=1
    IBBW=1
  elif [ "$RUN_TYPE" == "scatter" ]
  then
    tasks=$(( tasks/2 ))
    ppn=$(( tasks/nodes ))
    threads=$(( CORES_PER_SOCKET/ppn ))
    RUN_MASK=`gen_cpu_mask.py -s 2 -c $cores_per_node -t $threads -p ${ppn} --scatter`
    NUMACTL=
    MEMBW=2
    IBBW=2
  elif [ "$RUN_TYPE" == "scatter_ex" ]
  then
    RUN_MASK=`gen_cpu_mask.py -s 2 -c $cores_per_node -t $threads -p ${ppn} --scatter`
    NUMACTL=
    MEMBW=2
    IBBW=2
  elif [ "$RUN_TYPE" == "compact" ]
  then
    tasks=$(( tasks/2 ))
    ppn=$(( tasks/nodes ))
    threads=$(( CORES_PER_SOCKET/ppn ))
    RUN_MASK=`gen_cpu_mask.py -s 2 -c $cores_per_node -t $threads -p ${ppn} --compact`
    NUMACTL=
    MEMBW=1
    IBBW=2
  elif [ "$RUN_TYPE" == "interleaved" ]
  then
    RUN_MASK=`gen_cpu_mask.py -s 2 -c $cores_per_node -t $threads -p ${ppn} --normal`
    NUMACTL="numactl --interleave=0,1"
    MEMBW=0.69
    IBBW=1
  elif [ "$RUN_TYPE" == "mpi_profile" ]
  then
    RUN_MASK=`gen_cpu_mask.py -s 2 -c $cores_per_node -t $threads -p ${ppn} --normal`
    NUMACTL=
    MEMBW=1
    IBBW=1
    echo "Setting up variables for ipm mpi profiling"
    export I_MPI_STATS=ipm
    export I_MPI_STATS_FILE=ipm.out
  elif [ "$RUN_TYPE" == "low_bw" ]
  then
    RUN_MASK=`gen_cpu_mask.py -s 2 -c $cores_per_node -t $threads -p ${ppn} --normal`
    NUMACTL=
    MEMBW=1
    IBBW=0.5
    export PSM2_MTU=2
    echo "PSM2_MTU=$PSM2_MTU"
  else


    echo "unkown run type $RUN_TYPE"
    exit 1
  fi

########################################################################
# Setup OpenMP
########################################################################

  export OMP_NUM_THREADS=$threads
  export KMP_AFFINITY=verbose,scatter

########################################################################
# Display maxmium of benchmark parameters
########################################################################

  echo "------------------------------"
  echo "NNODES           : $nodes"
  echo "NODE LIST        : $SLURM_NODELIST"
  echo "NTASKS           : $tasks"
  echo "CORES per Socket : $CORES_PER_SOCKET"
  echo "Sockets          : $SOCKETS"
  echo "PPN              : $ppn"
  echo "THREADS/process  : $OMP_NUM_THREADS"
  echo "CPU_MASK         : $RUN_MASK"
  echo "FREQ             : $FREQ"
  echo "RUN_TYPE         : $RUN_TYPE"
  echo "CPU_GOVERNOR     : `cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor`"
  echo "MIN_FREQ         : `cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_min_freq`"
  echo "MAX_FREQ         : `cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_max_freq`"
  echo "CUR_FREQ         : `cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_cur_freq`"
  echo "THP              : `cat /sys/kernel/mm/transparent_hugepage/enabled`"
  echo "NUMA-BALANCING   : `cat /proc/sys/kernel/numa_balancing`"
  echo "KERNEL-VERSION   : `uname -a`"
  echo "FREE-MEM/NODE    : `free -g`"
  echo "PTRACE-SCOPE     : `cat /proc/sys/kernel/yama/ptrace_scope`"
  if [ -e /sys/devices/system/cpu/intel_pstate/no_turbo ]
  then
    echo "PSTATE_NOTURBO   : `cat /sys/devices/system/cpu/intel_pstate/no_turbo`"
  else
    echo "PSTATE_NOTURBO   : (intel pstate disabled)"
  fi
  echo "TURBO            : `cat /sys/devices/system/cpu/cpufreq/boost`"
  echo "------------------------------"
  echo "CPU_MASK in readable form:"
  gen_cpu_mask_amd.py -s 2 -c $cores_per_node  -d $RUN_MASK
  echo "------------------------------"

########################################################################
# Hardware specific tuning
########################################################################

export FORT_BUFFERED=true
export MALLOC_MMAP_MAX_=0
export MALLOC_TRIM_THRESHOLD_=-1

#IntelMPI 
export I_MPI_FABRICS=shm:dapl
#export I_MPI_FABRICS=shm:ofa
export I_MPI_DAPL_TRANSLATION_CACHE=1
#export I_MPI_DEBUG=5
export I_MPI_DAPL_UD=1

#Rome
export MKL_DEBUG_CPU_TYPE=5

#GPU tuning
###NGPU=4
###LISTGPU=`seq 0 $((NGPU-1))`
###export CUDA_VISIBLE_DEVICES=`echo $LISTGPU | sed 's/ /,/g'`

#TO BE CHECKED
#sudo nvidia-smi -ac 877,${FREQGPU}
#sudo nvidia-smi --auto-boost-default=ENABLED -i 0

#?
###export CUDA_DEVICE_MAX_CONNECTIONS=1

#IB tuning
#export UCX_TLS=rc_x,self,cuda_copy,gdr_copy,cuda_ipc
#export UCX_TLS=rc_x,self,gdr_copy,cuda_ipc
#export UCX_TLS=rc_x,self,cuda_ipc
export UCX_TLS=ud_x,sm,self
#export UCX_NET_DEVICES=all

########################################################################
# Prepare Working directory
########################################################################

  workdir=${FS}/DIR_${FREQ}_${nodes}N_${tasks}MPI_${threads}THRD_${RUN_TYPE}_${SLURM_JOBID}
  savedir=${SLURM_SUBMIT_DIR}/DIR_${FREQ}_${nodes}N_${tasks}MPI_${threads}THRD_${RUN_TYPE}_${SLURM_JOBID}
  mkdir -p ${workdir} || exit 1
  cd ${workdir} || exit 1
  #check that all nodes have access to directory
  clush -bw $SLURM_NODELIST " cd ${workdir}" || exit 1


  #copy script into working directory
  cp $0 .

  #dump envrionement
  env>env.file
  #dump limits
#  ulimit -s 262144
  ulimit -s unlimited
  ulimit -a >limits.file
 
  if [ ! -e "$BINARY" ]
  then
    echo "failed to locate $BINARY, please check path"
    exit 1
  fi
  
  #get path name
  exec_path=$( dirname $BINARY )
  bin=$(basename $BINARY)

  echo "Binary file is $bin, located in $exec_path"
 
########################################################################
# Display binary info
########################################################################

  cp $BINARY .

  echo "binary is $bin, dependencies:"
  ldd $bin || exit 1

########################################################################
# Generate data
########################################################################

#TESTCASE=large

DATA_DIR=$SLURM_SUBMIT_DIR/../DATA

#cp -rf $DATA_DIR/QE/${TESTCASE}/* .
cp -rf $DATA_DIR/* .


#case $TESTCASE in
#  small)
#    INPUT_FILE=ausurf.in
#  ;;
#  medium)
#    INPUT_FILE=Ta2O5-2x2xz-552.in
#  ;;
#  large)
#    INPUT_FILE=pw.in
#  ;;
#esac

#grir443.in
INPUT_FILE=grir443.in

echo "INPUT_FILE     : $INPUT_FILE"


#NSTEPS=200

#sed -i "s/electron_maxstep = 2/electron_maxstep = $NSTEPS/" $INPUT_FILE

#sed -i "s/outdir=\'./out/\'/outdir=\'./out/\',\ndisk_io=\'none\'/" $INPUT_FILE

#echo "TESTCASE     : $TESTCASE"

########################################################################
# Prepare run command
########################################################################

#for node in $(nodeset -e $SLURM_NODELIST)
#do
#        echo $node >> machinefile
#done

#create_rank_file.sh 2 $CORES_PER_SOCKET $ppn $threads off
#cat rank_file

echo "RUN !"

########################################################################
# Profiling
########################################################################
#export I_MPI_STATS=10

#  module load bpm/gcc-4.8/intelmpi-2018.1.163
#module load bpm/wo_IO/icc-2018.1.163/intelmpi-2018.1.163
#  export BPM_LEVEL=1

########################################################################
# Run command
########################################################################

  APP_CMD="$BINARY -inp $INPUT_FILE -ntg 2 -ndiag 200"
#MYDO='mpirun' 
#PW_COMMAND='pw.x' 
#PARA_OPT='-nk 4' 
#out_file='grir_443_'`date | awk -e '{print $2"_"$3"_"$6}'`'.out' 
#$MYDO   $PW_COMMAND $PARA_OPT -i grir443.in > $out_file
  APP_CMD="$BINARY -nk 4 -i $INPUT_FILE -ntg 2 -ndiag 200"

###  RUN_CMD="srun --distribution=plane=${ppn} -n ${tasks} --cpu_bind=verbose,mask_cpu:${RUN_MASK} ${NUMACTL} ${APP_CMD} "
  RUN_CMD="srun --distribution=plane=${ppn} -n ${tasks} --cpu_bind=verbose,mask_cpu:${RUN_MASK} ${NUMACTL} ${APP_CMD} "

#  export I_MPI_PIN_DOMAIN=[$RUN_MASK]
#  RUN_CMD="mpirun -n ${tasks} -rf rank_file -report-bindings ${APP_CMD}"
  echo "------------------------------"
  echo "RUN Command:"
  echo ${RUN_CMD}
  echo "------------------------------"
  date
  echo "------------------------------"
  time ${RUN_CMD} 2>&1 | tee output.log
  echo "------------------------------"
  date

########################################################################
# Cleanup
########################################################################

#TODO cleaning
#rm -rf checkpoint postprocessing

mkdir $savedir
cp $0 env_bench env.file output.log $savedir

########################################################################
# Study
########################################################################

cd $savedir
module load extrap_analyzer/0.1
bull_bench_instrument -p $ppn -t $[tasks * threads] -f $FREQ -c $IBBW -m $MEMBW 

