#!/bin/bash
#SBATCH -N 1
#SBATCH -n 4
#SBATCH -J Resnet50
#SBATCH --exclusive
#SBATCH -t 03:00:00
#SBATCH -o run_resnet50.%j
#SBATCH -p SKL-6148_GPU_hdr100_384gb_2666
#SBATCH -L scratch_na*1
##SBATCH -L scratch_lustre_ddn7990*1
#SBATCH --threads-per-core=1

#SBATCH -A rfp_2020_izum
##SBATCH -w spartan[412]
# Source benchmark environment


function check_clush() {
	clush -bw $SLURM_NODELIST $1 || fatal_error "clush command '$1' failed"
}

######################################
#  Define variables
######################################
ROOT=$SLURM_SUBMIT_DIR
SCRIPT_ADMIN=/home_nfs/script/admin
nodes=${SLURM_NNODES}
tasks=$SLURM_NTASKS
CORES_PER_SOCKET=`LANG=C lscpu|grep "Core(s) per socket:" |awk '{ print $NF }'`
SOCKETS=`LANG=C lscpu|grep "Socket(s):" |awk '{ print $NF }'`

cores_per_node=$(( CORES_PER_SOCKET*SOCKETS ))
ppn=$(( tasks/nodes ))
threads=$(( cores_per_node/ppn ))
export OMP_NUM_THREADS=1

num_gpu=$2
#########################################

if [ $1 -ne 16 ] && [ $1 -ne 32 ]; then
	echo "Arg 1 should be 16 or 32"
	exit
else
	PREC=$1
fi



export PYTVERSION=20.02-tf1-py3
#export PYTVERSION=20.02-tf2-py3
#export PYTVERSION=latest

export CONTAINER=docker://spartan10:5443/nvidia/tensorflow:${PYTVERSION}
#export CONTAINER=docker://spartan10:5443/bdsw/tensorflow-gpu:${PYTVERSION}
#export CONTAINER=docker://spartan0src/benchmarks_tf/perfzero/docker/Dockerfile_ubuntu_1804_tf_v2
export SINGULARITY_TMPDIR=/scratch_local/bpuddua   # local dir !! usually not big enough !!!
mkdir -p ${SINGULARITY_TMPDIR}


#####################################
# Set frequency
#####################################

#check_clush "sudo $SCRIPT_ADMIN/pwrb -T"
check_clush "sudo $SCRIPT_ADMIN/pwrb -N" 
#check_clush " sudo $SCRIPT_ADMIN/pwrb -f 2200000 "

#check_clush "sudo $SCRIPT_ADMIN/enable_boost"
check_clush "sudo $SCRIPT_ADMIN/disable_boost"
#check_clush "sudo $SCRIPT_ADMIN/enable_transparent_hugepages_rhel7 | sudo $SCRIPT_ADMIN/nr_hugepages 20000"
check_clush "sudo $SCRIPT_ADMIN/disable_transparent_hugepages_rhel7"

check_clush "sudo $SCRIPT_ADMIN/gpu_clock.sh 1"
#check_clush "sudo $SCRIPT_ADMIN/gpu_clock.sh 0"

FREQ_GR=$3

#nvidia-smi -ac 877,$FREQ_GR
#nvidia-smi -pm 0


########################################################################
# Hardware specific tuning
########################################################################
echo "---------HARD/SOFT TUNING---------------"
set -x


#export UCX_NET_DEVICES=mlx5_0:1
#export OMPI_MCA_coll_hcoll_enable=0
#export OMPI_MCA_pml=ucx
#export HCOLL_ENABLE_SHARP=0
#export UCX_RC_MAX_NUM_EPS=512
#export UCX_TLS=rc,rc_x,sm

ulimit -s unlimited

set +x
echo "------------------------------"
#

########################################################################
# Prepare Working directory
########################################################################

#DIR=$ROOT/$SLURM_JOB_PARTITION/FP$PREC/${nodes}N_${num_gpu}G_$SLURM_JOB_ID
DIR=/scratch_na/users/bpuddua/2020_Vega/TF/$SLURM_JOB_PARTITION/FP$PREC/${nodes}N_${num_gpu}G_$SLURM_JOB_ID
#DIR=/dev/shm/bpuddua/2020_Vega/TF/$SLURM_JOB_PARTITION/FP$PREC/${nodes}N_${num_gpu}G_$SLURM_JOB_ID
#DIR=/scratch_local/bpuddua/2020_Vega/TF/$SLURM_JOB_PARTITION/FP$PREC/${nodes}N_${num_gpu}G_$SLURM_JOB_ID
#DIR=$ROOT/$SLURM_JOB_PARTITION/FP$PREC/${nodes}N_${num_gpu}G_$SLURM_JOB_ID

mkdir -p $DIR
cd $DIR
cp $0 .
echo "EXEC_DIR= $DIR"

########################################################################
# Display maxmium of benchmark parameters
########################################################################

  echo "------------------------------"
  echo "NNODES           : $nodes"
  echo "NODE PARTITION   : $SLURM_JOB_PARTITION"
  echo "NODE LIST        : $SLURM_NODELIST"
  echo "NTASKS           : $tasks"
  echo "CORES per Socket :$CORES_PER_SOCKET"
  echo "Sockets          : $SOCKETS"
  echo "PPN              : $ppn"
  echo "THREADS/process  : $OMP_NUM_THREADS"
  echo "CPU_GOVERNOR     : `cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor`"
  echo "MIN_FREQ         : `cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_min_freq`"
  echo "MAX_FREQ         : `cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_max_freq`"
  echo "CUR_FREQ         : `cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_cur_freq`"
  echo "THP              : `cat /sys/kernel/mm/transparent_hugepage/enabled`"
  echo "NUMA-BALANCING   : `cat /proc/sys/kernel/numa_balancing`"
  echo "KERNEL-VERSION   : `uname -a`"
  echo "BOOST            : `cat /sys/devices/system/cpu/cpufreq/boost`"
  echo "FREE-MEM/NODE    : `free -g`"
  echo "PTRACE-SCOPE     : `cat /proc/sys/kernel/yama/ptrace_scope`"

  echo "GPU-CLOCK        : `nvidia-smi --query-gpu=clocks.applications.gr --format=csv | grep " MHz" | uniq`"
  echo "GPU-MEM          : `nvidia-smi --query-gpu=clocks.applications.mem --format=csv | grep " MHz" | uniq`"
  echo "GPU-DRIVER       : `nvidia-smi |grep Driver`"
  echo "------------------------------"



########################################################################
# RUN 
########################################################################
set -x
EXEC_DIR="/home_nfs_robin_ib/bpuddua/2020_Vega/tf-resnet50/benchmarks-master/scripts/tf_cnn_benchmarks"


#DATA_DIR=/scratch_lustre_ddn7990/bpajotb/2020_BSC/DL-Bench/data/train
#DATA_DIR=/scratch_na/users/bpuddua/2020_Vega/TF
DATA_DIR=/scratch_na/backup_aloy/scratch_gpfs_aloy/AI/MLPERF/datasets/ImageNet_2012/tf_records/train
#DATA1_DIR=/scratch_na/users/bpuddua/2020_Vega/TF/data/
#DATA2_DIR=/scratch_local/bpuddua/data_tf
#DATA2_DIR=/dev/shm/data_tf
#mkdir -p $DATA2_DIR

#cp $DATA_DIR/* $DATA2_DIR/
#cd $DATA2_DIR
CKPT_DIR=$DIR

set +x
echo "DATA_DIR= $DATA_DIR"

#batch_size=$3

if [ $PREC -eq 16 ]; then
CLIENT_CMD="$EXEC_DIR/tf_cnn_benchmarks.py \
--batch_size=640 \
--model=resnet50 \
 --optimizer=momentum \
 --variable_update=replicated \
 --nodistortions \
 --gradient_repacking=1 \
 --num_gpus=$num_gpu \
 --num_epochs=20 \
 --use_fp16\
 --data_dir=/datadir \
 --weight_decay=1e-4 \
 --train_dir=${CKPT_DIR}\
 --data_name=imagenet"
echo "BATCH_SIZE=640"
elif [ $PREC -eq 32 ]; then
CLIENT_CMD="$EXEC_DIR/tf_cnn_benchmarks.py \
--batch_size=256 \
--model=resnet50 \
 --optimizer=momentum \
 --variable_update=replicated \
 --nodistortions \
 --gradient_repacking=1 \
 --num_gpus=$num_gpu \
 --nouse_fp16 \
 --weight_decay=1e-4 \
 --data_dir=/datadir\
 --num_epochs=20 \
 --train_dir=${CKPT_DIR}\
 --data_name=imagenet"
fi

if [ $num_gpu -eq 4 ]; then  
export CUDA_VISIBLE_DEVICES=0,1,2,3
elif [ $num_gpu -eq 2 ]; then  
export CUDA_VISIBLE_DEVICES=0,2
elif [ $num_gpu -eq 1 ]; then  
export CUDA_VISIBLE_DEVICES=0
fi
#MASK_CPU="0xFFC,0XFFC000,0xFFC000000,0xFFC000"
#RUN_CMD="srun  -n $tasks --mpi=pmi2 --cpu_bind=rank,map_cpu:0,10,20,30 \
   #singularity exec --nv $CONTAINER python3 ${CLIENT_CMD}"
#RUN_CMD="srun -n 4 --mpi=pmi2 --cpu_bind=mask_cpu:$MASK_CPU singularity exec -B $DATA_DIR:/datadir --nv $CONTAINER  python3 ${CLIENT_CMD}"
RUN_CMD="singularity exec -B $DATA_DIR:/datadir --nv $CONTAINER python3 ${CLIENT_CMD}"



echo "VISIBLE_DEVICES=   $CUDA_VISIBLE_DEVICES"
echo "RUN_CMD=    $RUN_CMD"


/usr/bin/time $RUN_CMD 2>&1 | tee output.log

export RES_DIR=$ROOT/run_scratch/FP$PREC/${nodes}N_${num_gpu}G_$SLURM_JOB_ID
mkdir $RES_DIR
mv $ROOT/run_resnet50.$SLURM_JOB_ID $DIR

mv $DIR/* $RES_DIR 
rm -rf /scratch_local/bpuddua
#rm -rf /dev/shm/bpuddua


